TITLE add_2022.asm: dal compito di Calcolatori del 22 aprile 2022

comment *
	Algoritmo per l'addizione intera in assembler 8086 di parole a 8 bit
	basato su operazioni di AND, XOR e SHIFT LEFT su registri di macchina
	(N.B. si finge che il processore sia privo dell'apposita istruzione di somma ADD)

		data creazione: 24 aprile 2022
		ultima revisione: 24 aprile 2022
*


;-----------------------------------------------------------------
; Definizione costanti
CR EQU 13                      ; carriage return
LF EQU 10                      ; line feed
DOLLAR EQU '$'


;-----------------------------------------------------------------
;    M  A  C  R  O
;-----------------------------------------------------------------

display macro xxxx           ; N.B. ogni stringa deve terminare con '$' 
        push dx
	    push ax
	    mov dx,offset xxxx
	    mov ah,9
	    int 21h
	    pop ax
        pop dx
endm


;-----------------------------------------------------------------
;
PILA SEGMENT STACK 'STACK'     ; definizione del segmento di stack
      DB      64 DUP('STACK')  ; lo stack e' riempito con la stringa 'stack'
                               ; per identificarlo meglio in fase di debug
PILA ENDS                      

;-----------------------------------------------------------------
;
DATI SEGMENT PUBLIC 'DATA'    ; definizione del segmento dati

CRLF                 db CR,LF,DOLLAR
SEP                  db ':',DOLLAR
CROSS                db ' + ',DOLLAR
EQUAL                db ' = ',DOLLAR

messaggio1           db "*** ADDIZIONE INTERA SEQUENZIALE con algoritmo basato su AND, XOR e SHIFT ***",DOLLAR
messaggio3           db "risultato calcolato con l'istruzione ADD del processore:  ",DOLLAR
messaggio4           db "risultato calcolato con l'algoritmo AND, XOR & SHIFT   :  ",DOLLAR

addendo1             db 186 ;255;157;11; chiamato "a" nel compito
addendo2             db 91  ;255;234;14; chiamato "b" nel compito
somma                dw ?

; per conversione del risultato in ASCII e stampa a video
max_strlen equ 16
stringa_ax           db max_strlen dup (?),' $'
alfabeto_numerazione db "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"

DATI ENDS  
 
;=================================================================

CSEG1 SEGMENT PUBLIC 'CODE'

MAIN proc far
        ASSUME CS:CSEG1,DS:DATI,SS:PILA,ES:NOTHING;

        MOV AX,SEG DATI
        MOV DS,AX
		
		; stampa a video messaggio iniziale
		display crlf
        display messaggio1
		display crlf		
		
		; lettura operandi da memoria
		xor dh,dh
		mov dl, addendo1 ; addendo1 sta nel byte meno significativo di DX
        xor bh,bh		
        mov bl, addendo2 ; addendo2 sta nel byte meno significativo di BX

        ;****** 1. USO DELL'ISTRUZIONE 8086 'ADD' *****

        ; stampa a video operandi
		display crlf
		display messaggio3  

		mov ax,dx
        call near ptr display_ax
        display cross
		mov ax,bx
		call near ptr display_ax
        display equal
                
        mov ax,dx ; N.B. Non si potrebbe sommare AX con DL ("syntax error"): per questo sopra si e' azzerato DH (estensione di segno)
        add ax,bx ; in ax la somma di DX e BX (versione estesa di BL)

        ; stampa a video risultato, che e' in AX
		call near ptr display_ax              
		display crlf



        ;****** 2. MULTIPLICAZIONE con algoritmo basato su AND, XOR e SHIFT *****
              
        ; stampa a video operandi
        display messaggio4
		
		mov ax,dx
        call near ptr display_ax
        display cross
		mov ax,bx
		call near ptr display_ax
        display equal
     
	    ; inizializzazione: s <- a XOR b, r <- a AND b
        mov al,dl 
		xor al,bl ; s = AL
		mov cl,dl
		and cl,bl ; r = CL
		xor ah,ah ; l'eventuale carry della somma sara' posto in AH
		
        ciclo_somma: 	
		    test cl,10000000B ; controllo se il bit che si perdera' nello shift left sia 1,
			                  ; nel qual caso bisogna settare a 1 il bit di carry del risultato
			jz shift_left
		set_carry:
		    mov ah,00000001B
		shift_left:
		    shl cl,1  ; r << 1
		update:
			mov ch,al ; salvataggio di s per poterlo usare nell'AND
		    xor al,cl ; aggiornamento di s: s <- r XOR s (r e' quello appena shiftato)
			and cl,ch ; aggiornamento di r: r <- r AND s (s e' quello non aggiornato)
			jnz ciclo_somma ; si esce se r=0

        ; stampa a video risultato, che e' in AX (in AH l'eventuale carry, in AL la somma su 8 bit)
		call near ptr display_ax              
		display crlf

        ;***************************************************************

	exit: 
        MOV AH,4CH                 ; ritorno al DOS
        INT 21H

main endp



; routine di conversione di un numero a 16 bit (il cui valore e' posto nello stack)
; nell'equivalente in una base prefissata, espresso in ASCII e posto
; in una stringa di N caratteri, il cui indirizzo e' anch'esso inserito nello stack
; prima della chiamata. Il valore di ritorno N rimpiazza il valore immesso nello stack
; dal chiamante, che indica la base in cui convertire il numero
;
bin2bascii proc near
        push ax
        push bx
        push cx
        push dx
        push si
        push di
        push bp
        pushf

        mov bp,sp
        add bp,16                          ; rimuove l'effetto dei push qui sopra
        mov ax,SS:[bp+6]                   ; numero da convertire (e' sepolto sotto 3 words)

        mov bx,[bp+2]
        mov cx,0
again:  xor dx,dx
        div bx                             ; in DX il resto, in AX il quoziente di [DX:AX]/[BX]
                                           ; N.B. con or dx,0030h si renderebbe ASCII il contenuto di dx
        mov si,dx
        mov dl,alfabeto_numerazione[si]    ; si prende il simbolo di numerazione dall'array apposito
        xor dh,dh
        push dx
        inc cx
        cmp ax,0
        je save_result
        jmp again
save_result:
        mov di,0
        mov bx,[bp+4]                      ; offset della stringa destinazione
cycle:  pop dx
        mov [bx][di],dl
        inc di
        loop cycle
        mov [bp+2],di                      ; numero di caratteri nella stringa destinazione

        popf
        pop bp
        pop di
        pop si
        pop dx
        pop cx
        pop bx
        pop ax

        ret

bin2bascii endp

; routine di conversione e stampa a video formattata  
; di un numero intero non negativo posto nel registro AX
;
display_ax proc near 
        push ax
        push bx

        push ax
        mov ax,offset stringa_ax
        push ax
        mov ax,10
        push ax
        call near ptr bin2bascii
        pop bx                       ; in bx il numero di caratteri della stringa
        add sp,4                                     

        mov stringa_ax[bx],'$'        ; $ per poter stampare
        display stringa_ax

        pop bx
        pop ax
        ret

display_ax endp



cseg1 ends

END MAIN                     ; il programma comincia all'indirizzo di MAIN

                         
