TITLE bb3_UTM.asm (cfr. compito Calcolatori febbraio 2020)

comment *
#              Busy beaver a tre stati. Fonti: Dudeney, Wolfram, Nahin, Wikipedia.
#
#              Tre esempi di macchine (tra le 2^24 = 16M possibili) sono i seguenti
#              (tabelle scritte in forma di quintuple di Turing):
#
#              BB-3a   BB-3b   BB-3s 
#              ------  ------  ------
#              0A:B1+  0A:B1+  0A:B1+
#              0B:C0+  0B:A1-  0B:B1-
#              0C:C1-  0C:B1-  0C:C1-
#              1A:D1+  1A:C1-  1A:D1+
#              1B:B1+  1B:B1+  1B:C0+
#              1C:A1-  1C:D1+  1C:A1-
#
#              La 3a e la 3b scrivono 6 "1", con 14 e 13 passi rispettivamente.
#              La s scrive 5 "1", con 21 passi.
#
#              Si usa la seguente codifica: A=0 (stato iniziale), 
#                                           B=1 
#                                           C=2
#                                           D=3 (stato di terminazione)
# 
#              Convenzione per la testina: + = una cella a dx (incremento)
#                                          - = una cella a sx (decremento)
#
#              Questa e' la versione UTM (Universal Turing Machine), in cui la specifica
#              completa della macchina e' contenuta in memoria (anche se non sullo stesso
#              nastro dei dati) all'inizio dell'elaborazione. Per far girare un'altra
#              macchina a quattro stati, basta cambiare la stringa della specifica.
#
#              ultima modifica: 21 febbraio 2020
*

;-----------------------------------------------------------------
; Definizione costanti
CR EQU 13                      ; carriage return
LF EQU 10                      ; line feed
DOLLAR EQU '$'


;-----------------------------------------------------------------
;    M  A  C  R  O
;-----------------------------------------------------------------

display macro xxxx                 ; N.B. ogni stringa deve terminare con '$' 
    push dx
	push ax
	mov dx,offset xxxx
	mov ah,9
	int 21h
	pop ax
    pop dx
endm



;-----------------------------------------------------------------
;
PILA SEGMENT STACK 'STACK'     ; definizione del segmento di stack
      DB      16 DUP('STACK')  ; lo stack e' riempito con la stringa 'stack'
                               ; per identificarlo meglio in fase di debug
PILA ENDS                      

;-----------------------------------------------------------------
;
DATI SEGMENT PUBLIC 'DATA'         ; definizione del segmento dati

tapelen equ 32

head_init equ tapelen/2

tape   db tapelen dup ('0') ; nastro di Turing (ma di lunghezza finita, e sufficiente per il programma)
       db DOLLAR            ; serve per il display del nastro

Anum equ 0 ; codici numerici per gli stati
Dnum equ 3

index dw 0,6,12 ; = stato presente x 6, con 
                ; 6 = valori associati a ciascuno stato presente nella tabella = 3 x 2 (ingressi possibili))
				
Triplets_num equ 6 ; = #stati x #ingressi possibili
Machine_len equ 3 * Triplets_num ; # totale simboli nella specifica di una macchina
                                 ; Tripletta = (stato futuro, simbolo scritto, direzione)
;Symbolic_Machine db "B1+D1+C0+B1+C1-A1-",DOLLAR ; BB-3a
;Symbolic_Machine db "B1+C1-A1-B1+B1-D1+",DOLLAR ; BB-3b
Symbolic_Machine db "B1+D1+B1-C0+C1-A1-",DOLLAR ; BB-3s
ASCII_Machine db Machine_len dup (?),DOLLAR
Numerical_Machine db Machine_len dup (?)

CRLF                 db CR,LF,DOLLAR
COLON                db ": ",DOLLAR
VSLASH                db "| ",DOLLAR
SEMICOLON            db " ; ",DOLLAR
ZERO                 db "0",DOLLAR
ZEROSPC              db "0 ",DOLLAR
ZEROZERO             db "00 : ",DOLLAR

max_strlen equ 16
stringa_ax           db max_strlen dup (?),' $'
alfabeto_numerazione db "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"

DATI ENDS  
 

;=================================================================

CSEG SEGMENT PUBLIC 'CODE'

MAIN proc far
        ASSUME CS:CSEG,DS:DATI,SS:PILA,ES:NOTHING;

        MOV AX,DATI
        MOV DS,AX

        ; conversione della rappresentazione simbolica della Turing Machine in quella ASCII
		display crlf	
		display crlf	
        display Symbolic_Machine
		display crlf		
        xor si,si
        mov cx, Triplets_num
S2A_Convert:
        mov al,Symbolic_Machine[si]
		sub al,'A'
		add al,'0'
		mov ASCII_Machine[si],al
        mov al,Symbolic_Machine[si+1]
		mov ASCII_Machine[si+1],al		
        mov al,Symbolic_Machine[si+2]
		cmp al,'+'
        jne minus
        mov ASCII_Machine[si+2],'1'
		jmp adjourn
minus:
        mov ASCII_Machine[si+2],'0'
adjourn:
		add si,3
        loop S2A_Convert	
		display ASCII_Machine
		display crlf	



        ; conversione della rappresentazione ASCII della Turing Machine in quella numerica
        xor si,si
        mov cx, Machine_len
A2N_Convert:	
        mov al,ASCII_Machine[si]
		sub al,'0'
		mov Numerical_Machine[si],al
		inc si
		loop A2N_Convert	
		
start:
		display crlf
		display ZEROZERO
		display ZEROSPC		
        display VSLASH	
		display tape        ; mostra lo stato iniziale del nastro
		display SEMICOLON		
		mov ax, head_init
        call near ptr display_ax	; mostra la posizione iniziale della testina di lettura/scrittura		
		display crlf

		xor cx,cx           ; cx conta il numero di steps della macchina
	    mov bx, offset tape ; qui bx punta alla prima cella del nastro
		add bx, head_init   ; qui bx punta alla cella di partenza del nastro

		mov si,Anum 		; si contiene lo stato presente (lo stato iniziale e' A = 0)
 BB_cycle:
		shl si,1		    ; raddoppio (temporaneamente) si
        mov di,index[si]    ; index e' un vettore di word, dunque di = index[si*2]
		shr si,1            ; ripristino il valore di si      
 		mov ah,[bx]         ; lettura della cella: va in ah 	
        cmp ah,'0'
		je input_ZERO
	input_ONE:    
        mov al,Numerical_Machine[di+3]
		mov dh,ASCII_Machine[di+4]
		mov dl,Numerical_Machine[di+5]
		jmp update
	input_ZERO:	
        mov al,Numerical_Machine[di]    ; parcheggio lo stato futuro in al: lo mettero' piu' giu' in si
		mov dh,ASCII_Machine[di+1]      ; simbolo scritto
		mov dl,Numerical_Machine[di+2]  ; spostamento testina
	update:	
        mov [bx],dh	
		cmp dl,0
		je decrement
		increment:
			inc bx
			jmp set_new_state
		decrement:
			dec bx
		set_new_state:	
        xor ah,ah			
		mov si,ax 		
		inc cx
		; aggiornamento del display
		cmp cx,9
		jg no_leading_zero
		display ZERO
no_leading_zero:
		mov ax,cx
        call near ptr display_ax	; mostra il numero di steps gia' eseguiti dalla macchina
        display COLON	
		mov ax,si 	
		call near ptr display_ax	; mostra lo stato presente
        display VSLASH		
        display tape                ; mostra lo stato attuale del nastro
		display SEMICOLON				
		mov ax,bx
		sub ax, offset tape
        call near ptr display_ax	; mostra la posizione corrente della testina di lettura/scrittura
		display crlf		
termination_check:		
		cmp si,Dnum
		je terminate
		jmp BB_cycle		
terminate:                                                          								  			
        display crlf	
		display crlf		
		
exit: 
        MOV AH,4CH  ; ritorno al DOS
        INT 21H

main endp




; routine di conversione di un numero a 16 bit (il cui valore e' posto nello stack)
; nell'equivalente in una base prefissata, espresso in ASCII e posto
; in una stringa di N caratteri, il cui indirizzo e' anch'esso inserito nello stack
; prima della chiamata. Il valore di ritorno N rimpiazza il valore immesso nello stack
; dal chiamante, che indica la base in cui convertire il numero
bin2bascii proc near
        push ax
        push bx
        push cx
        push dx
        push si
        push di
        push bp
        pushf

        mov bp,sp
        add bp,16                          ; rimuove l'effetto dei push qui sopra
        mov ax,SS:[bp+6]                   ; numero da convertire (e' sepolto sotto 3 words)

        mov bx,[bp+2]
        mov cx,0
        mov dx,0
again:  div bx                             ; in DX il resto, in AX il quoziente di [DX:AX]/[BX]
        ;or dx,0030h                       ; si rende ASCII il contenuto di dx
        mov si,dx
        mov dl,alfabeto_numerazione[si]    ; si prende il simbolo di numerazione dall'array apposito
        xor dh,dh
        push dx
        inc cx
        cmp ax,0
        je save_result
        xor dx,dx
        jmp again
save_result:
        mov di,0
        mov bx,[bp+4]                      ; offset della stringa destinazione
cycle:  pop dx
        mov [bx][di],dl
        inc di
        loop cycle
        mov [bp+2],di                      ; numero di caratteri nella stringa destinazione

        popf
        pop bp
        pop di
        pop si
        pop dx
        pop cx
        pop bx
        pop ax

        ret

bin2bascii endp


display_ax proc near 
        push ax
        push bx

        push ax
        mov ax,offset stringa_ax
        push ax
        mov ax,10
        push ax
        call near ptr bin2bascii
        pop bx                       ; in bx il numero di caratteri della stringa
        add sp,4                                     

        mov stringa_ax[bx],' '       ; spazio di formattazione
        mov stringa_ax[bx+1],'$'     ; $ per poter stampare
        display stringa_ax

        pop bx
        pop ax
        ret

display_ax endp



cseg ends

END MAIN                     ; il programma comincia all'indirizzo di MAIN






   

                      
