TITLE divis__7.asm: dal compito 8/7/22

comment *
Programma che testa la divisibilità per q=7 di un intero a 16 bit,
senza fare uso di divisioni, ma sfruttando invece look-up tables ottenute
(a mano) attraverso l'aritmetica modulare.

Data ultima modifica: 5 luglio 2022.
*

;-----------------------------------------------------------------
; Definizione costanti
CR EQU 13                      ; carriage return
LF EQU 10                      ; line feed
DOLLAR EQU '$'

;-----------------------------------------------------------------
;    M  A  C  R  O
;-----------------------------------------------------------------

display macro xxxx                 ; N.B. ogni stringa deve terminare con '$' 
    push dx
	push ax
	mov dx,offset xxxx
	mov ah,9
	int 21h
	pop ax
    pop dx
endm


;-----------------------------------------------------------------
;
PILA SEGMENT STACK 'STACK'     ; definizione del segmento di stack
      DB      16 DUP('STACK')  ; lo stack e' riempito con la stringa 'stack'
                               ; per identificarlo meglio in fase di debug
PILA ENDS                      

;-----------------------------------------------------------------
;
DATI SEGMENT PUBLIC 'DATA'         ; definizione del segmento dati

n equ 16 ; lunghezza di rappresentazione dell'intero da testare

p dw 0E54h ; =3668, usato per la simulazione del programma al secondo punto del compito: E' divisibile per 7 (sequenza SP: 00000130001253000)
;p dw 32743 ; non divisibile per 7
;p dw 59661 ; = 8523 x 7
q DW 7     ; modulo utilizzato per il test



r_0 db 0 ; la macchina parte da questo stato

; realizzazione in memoria della tabella dello stato futuro (in funzione di stati presente e dell'ingresso)
table_pointers dw 7 dup(?)           ; vettore dei puntatori (N.B. ciascuno di 2 byte!) ai sette vettori degli stati futuri
                                     ; vettori degli stati futuri (ce n'e' uno per ciascuno stato presente; come indice all'interno del vettore si usera' l'ingresso, 0 o 1)
future_states_from_0 db 0,1      
future_states_from_1 db 2,3      ; si legge cosi': "nello stato 1, se l'ingresso e' 0  allora lo stato futuro e' 4, se e' 1 lo s.f. e' 3"
future_states_from_2 db 4,5  
future_states_from_3 db 6,0
future_states_from_4 db 1,2
future_states_from_5 db 3,4
future_states_from_6 db 5,6    

prestates  db n+1 dup (?),DOLLAR  ; stringa ASCII degli stati (presenti) attraversati dalla macchina (riempita dal programma e alla fine stampata a video)
                                  ; N.B. sono n+1 perche' c'e' lo stato iniziale, e poi lo stato cambia n volte per via degli n bit dell'input    


SP_msg  db "Residui attraversati:  ",DOLLAR
out1_msg db "p = ",DOLLAR
out2_msg db "q = ",DOLLAR
out3_msg db "Divisibilita' di p per q? ",DOLLAR
out4a_msg db "NO! ",DOLLAR
out4b_msg db "SI'! ",DOLLAR

CRLF      db CR,LF,DOLLAR ; per formattazione output

; per conversione del risultato in ASCII e stampa a video
max_strlen equ 16
stringa_ax           db max_strlen dup (?),' $'
alfabeto_numerazione db "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"


DATI ENDS  
 

;=================================================================

CSEG SEGMENT PUBLIC 'CODE'

MAIN proc far
        ASSUME CS:CSEG,DS:DATI,SS:PILA,ES:NOTHING;

        MOV AX,DATI
        MOV DS,AX
		
		display crlf
		display out1_msg
		mov ax,p
		call near ptr display_ax 
		display crlf		
		display out2_msg
		mov ax,q
		call near ptr display_ax		
		display crlf
	
	inizializzazione:		
		mov bx,offset future_states_from_0
		mov table_pointers[0],bx
		add bx,2                ; N.B. il numero di stati futuri (e dunque anche la lunghezza del vettore future_states_from_0) e' 2
		mov table_pointers[2],bx ; qui bx contiene l'offset di future_states_from_1; N.B. table_pointers va incrementato di due byte, perche' e' un vettore di words
		add bx,2
		mov table_pointers[4],bx ; qui bx contiene l'offset di future_states_from_2
		add bx,2
		mov table_pointers[6],bx ; qui bx contiene l'offset di future_states_from_3
		add bx,2
		mov table_pointers[8],bx ; qui bx contiene l'offset di future_states_from_4
		add bx,2
		mov table_pointers[10],bx ; qui bx contiene l'offset di future_states_from_5
		add bx,2
		mov table_pointers[12],bx ; qui bx contiene l'offset di future_states_from_6	
				          
		xor bh,bh ; Per rappresentare stato presente e stato futuro bastebbe la parte bassa dei registri (8 bit),  
		xor dh,dh ; ma i registri dovranno essere usati anche interi (16 bit), e per questo si fa l'estensione di segno

		mov cx,n ; inizializzazione del contatore con il numero di iterazioni da eseguire
		xor si,si ; inizializzazione a zero dell'indice per il riempimento del vettore degli stati presenti

	    mov bl,r_0 ; inizializzazione dello stato presente		
		mov ax,p  ; ax contiene inizialmente p, e viene via via shiftato a sinistra di una posizione per testare i vari bit di p a partire dal MSB
		
	funzionamento:	
	    ; INGRESSO
		mov bp,ax ; il registro bp conterra' l'input b_t e sara' usato come indice nella look-up table
		and bp,1000000000000000B
		jz zero
		mov bp,1 ; il bit b_t e' 1
		zero: ; se si salta qui, il bit b_t e' 0

		
	    ; STATO PRESENTE
		mov dl,bl   ; dl contiene qui lo stato presente in formato binario
		add dl, '0' ; dl contiene qui lo stato presente in formato ASCII
		mov prestates[si],dl ; salvataggio dell'uscita (formato ASCII) nella stringa "prestates"	
		
		; STATO FUTURO
        shl bx,1 ; lo stato presente, che e' in bx, viene raddoppiato perche' usato nell'istruzione successiva come indice nel vettore di words	
		mov di,table_pointers[bx] ; di contiene qui l'offset del vettore degli stati futuri dallo stato presente
		add di,bp ; il puntatore e' posizionato sul giusto stato futuro, servendosi dell'input in bp come indice (di <- di+bp)		
		mov dl,[di] ; lettura dello stato futuro, che alla prossima iterazione sara' in bl (e, nella versione a 16 bit, in bx)
		
	aggiornamento:	
		shl ax,1 ; ax preparato per testare il prossimo bit	
        mov bx,dx
		inc si		
		dec cx
		jnz funzionamento
		
	; display dei risultati e uscita dal programma
	
	mov dl,bl   ; dl contiene qui lo stato presente in formato binario
	add dl, '0' ; dl contiene qui lo stato presente in formato ASCII
	mov prestates[si],dl ; scrittura dell'ultimo, ovvero (n+1)-simo, stato	
	stampa_a_video:	
	    display SP_msg
		display prestates		
		display crlf		
		
		display OUT3_msg		
        cmp bx,0
        je divisibile
		nondivisibile: 
		display OUT4a_msg
		display crlf		
		jmp exit
		divisibile:
		display OUT4b_msg
		display crlf
		
    exit: 
        MOV AH,4CH              ; ritorno al DOS
        INT 21H

main endp


; routine di conversione di un numero a 16 bit (il cui valore e' posto nello stack)
; nell'equivalente in una base prefissata, espresso in ASCII e posto
; in una stringa di N caratteri, il cui indirizzo e' anch'esso inserito nello stack
; prima della chiamata. Il valore di ritorno N rimpiazza il valore immesso nello stack
; dal chiamante, che indica la base in cui convertire il numero
;
bin2bascii proc near
        push ax
        push bx
        push cx
        push dx
        push si
        push di
        push bp
        pushf

        mov bp,sp
        add bp,16                          ; rimuove l'effetto dei push qui sopra
        mov ax,SS:[bp+6]                   ; numero da convertire (e' sepolto sotto 3 words)

        mov bx,[bp+2]
        mov cx,0
again:  xor dx,dx
        div bx                             ; in DX il resto, in AX il quoziente di [DX:AX]/[BX]
                                           ; N.B. con or dx,0030h si renderebbe ASCII il contenuto di dx
        mov si,dx
        mov dl,alfabeto_numerazione[si]    ; si prende il simbolo di numerazione dall'array apposito
        xor dh,dh
        push dx
        inc cx
        cmp ax,0
        je save_result
        jmp again
save_result:
        mov di,0
        mov bx,[bp+4]                      ; offset della stringa destinazione
cycle:  pop dx
        mov [bx][di],dl
        inc di
        loop cycle
        mov [bp+2],di                      ; numero di caratteri nella stringa destinazione

        popf
        pop bp
        pop di
        pop si
        pop dx
        pop cx
        pop bx
        pop ax

        ret

bin2bascii endp

; routine di conversione e stampa a video formattata  
; di un numero intero non negativo posto nel registro AX
;
display_ax proc near 
        push ax
        push bx

        push ax
        mov ax,offset stringa_ax
        push ax
        mov ax,10
        push ax
        call near ptr bin2bascii
        pop bx                       ; in bx il numero di caratteri della stringa
        add sp,4                                     

        mov stringa_ax[bx],'$'        ; $ per poter stampare
        display stringa_ax

        pop bx
        pop ax
        ret

display_ax endp



cseg ends

END MAIN                     ; il programma comincia all'indirizzo di MAIN






   

                      
