TITLE morsenc.asm: dal compito 9/9/22

comment *
Programma che codifica in Morse una stringa alfabetica data.

Data ultima modifica: 14 settembre 2022.
*

;-----------------------------------------------------------------
; Definizione costanti
CR EQU 13                      ; carriage return
LF EQU 10                      ; line feed
DOLLAR EQU '$'

;-----------------------------------------------------------------
;    M  A  C  R  O
;-----------------------------------------------------------------

display macro xxxx                 ; N.B. ogni stringa deve terminare con '$' 
    push dx
	push ax
	mov dx,offset xxxx
	mov ah,9
	int 21h
	pop ax
    pop dx
endm


;-----------------------------------------------------------------
;
PILA SEGMENT STACK 'STACK'     ; definizione del segmento di stack
      DB      16 DUP('STACK')  ; lo stack e' riempito con la stringa 'stack'
                               ; per identificarlo meglio in fase di debug
PILA ENDS                      

;-----------------------------------------------------------------
;
DATI SEGMENT PUBLIC 'DATA'         ; definizione del segmento dati

ALFA_ASCII db "NICCOLO AND ALICE",'$' ; N.B. nel codice si assume che le lettere siano tutte MAIUSCOLE (infatti si sottrae 'A')
n equ $-1-ALFA_ASCII ; lunghezza della stringa da codificare (il -1 evita di contare il '$', che serve solo per la stampa a video)

m equ 5*n+1 ; lunghezza max della stringa codificata: 4 caratteri per lettera (incluso lo spazio) + separatore ("2") lettera + simbolo ("3") di fine stringa

ALFA_MORSE db m dup(?)

MORSE_TABLE label byte ; le x servono ad avere una codifica di 5 caratteri per lettera
_A_ db "012xx" ;   0 (offset nella tabella)
_B_ db "10002" ;   5 (= 1*5, con 1 = 'B'-'A')
_C_ db "10102" ;  10 (= 2*5)
_D_ db "1002x" ;  15 (= 3*5)
_E_ db "02xxx" ;  20 (= 4*5)
_F_ db "00102" ;  25 (= 5*5)
_G_ db "1102x" ;  30 (= 6*5)
_H_ db "00002" ;  35 (= 7*5)
_I_ db "002xx" ;  40 (= 8*5)
_J_ db "01112" ;  45 (= 9*5)
_K_ db "1012x" ;  50 (=10*5)
_L_ db "01002" ;  55 (=11*5)
_M_ db "112xx" ;  60 (=12*5)
_N_ db "102xx" ;  65 (=13*5)
_O_ db "1112x" ;  70 (=14*5)
_P_ db "01102" ;  75 (=15*5)
_Q_ db "11012" ;  80 (=16*5)
_R_ db "0102x" ;  85 (=17*5)
_S_ db "0002x" ;  90 (=18*5)
_T_ db "12xxx" ;  95 (=19*5)
_U_ db "0012x" ; 100 (=20*5)
_V_ db "00012" ; 105 (=21*5)
_W_ db "0112x" ; 110 (=22*5)
_X_ db "10012" ; 115 (=23*5)
_Y_ db "10112" ; 120 (=24*5)
_Z_ db "11002" ; 125 (=25*5)

CRLF      db CR,LF,DOLLAR ; per formattazione output
ASCSTR    db "ALFA_ASCII = $"
MORSTR    db "ALFA_MORSE = $"


DATI ENDS  
 

;=================================================================

CSEG SEGMENT PUBLIC 'CODE'

MAIN proc far
        ASSUME CS:CSEG,DS:DATI,SS:PILA,ES:NOTHING;

        MOV AX,DATI
        MOV DS,AX

    stampa_input:		
		display crlf
		display ASCSTR
		display ALFA_ASCII

        mov cx,n
		mov dl,5 ; = numero di caratteri per lettera (da usare con l'istruzione MUL)
		mov si, offset ALFA_ASCII
		mov di, offset ALFA_MORSE
	scansione_stringa_input: ; loop esterno: scandisce uno per uno i caratteri da codificare 
        mov al,[si]
		cmp al,20h ; lo spazio (20h = 32) segna la fine di una parola: al suo posto va messo un '2'
		je scrivi_2
		sub al,'A' ; riduco il carattere ASCII (lettera MAIUSCOLA) a un numero da 0 a 25 : 'A'=0, 'B'=1, etc.
		mul dl ; AX = al * dl (al e' un operando implicito). Es. 'P'<->15*5=75 (vedi segmento dati)
		mov bx, ax ; sposto l'offset ax nel registro bx per poter utilizzare quest'ultimo come offset nella tabella
	scrivi_lettera: ; loop interno: scrive in memoria la sottostringa corrispondente a una lettera
		mov ah, MORSE_TABLE[bx]
		cmp ah,'2'
		je scrivi_2
		mov [di],ah
		inc di
		inc bx
		jmp scrivi_lettera
    scrivi_2:
		mov [di],ah
		inc di	
	aggiorna:
		inc si ; punta al nuovo carattere di ALFA_ASCII
        dec cx
		jnz scansione_stringa_input
    scrivi_3:
	    mov ah,'3'
		mov [di],ah
		inc di
	    mov ah,'$'   ; queste ultime due istruzioni servono solo ad aggiungere il '$' per poter stampare a video ALFA_MORSE
		mov [di],ah		

    stampa_output:
		display crlf	
		display MORSTR	
		display ALFA_MORSE

		display crlf
		
    exit: 
        MOV AH,4CH              ; ritorno al DOS
        INT 21H

main endp


cseg ends

END MAIN                     ; il programma comincia all'indirizzo di MAIN






   

                      
